@@ -1,8 +1,4 @@
 /*
- * $HeadURL: https://svn.apache.org/repos/asf/httpcomponents/httpcore/tags/4.0.1/httpcore-nio/src/main/java/org/apache/http/impl/nio/codecs/LengthDelimitedDecoder.java $
- * $Revision: 744538 $
- * $Date: 2009-02-14 18:20:23 +0100 (Sat, 14 Feb 2009) $
- *
  * ====================================================================
  * Licensed to the Apache Software Foundation (ASF) under one
  * or more contributor license agreements.  See the NOTICE file
@@ -41,30 +37,27 @@ import org.apache.http.nio.FileContentDecoder;
 import org.apache.http.nio.reactor.SessionInputBuffer;
 
 /**
- * Content decoder that cuts off after a defined number of bytes. This class 
- * is used to receive content of HTTP messages where the end of the content 
- * entity is determined by the value of the <code>Content-Length header</code>. 
+ * Content decoder that cuts off after a defined number of bytes. This class
+ * is used to receive content of HTTP messages where the end of the content
+ * entity is determined by the value of the <code>Content-Length header</code>.
  * Entities transferred using this stream can be maximum {@link Long#MAX_VALUE}
- * long. 
+ * long.
  * <p>
- * This decoder is optimized to transfer data directly from the underlying 
- * I/O session's channel to a {@link FileChannel}, whenever 
- * possible avoiding intermediate buffering in the session buffer. 
+ * This decoder is optimized to transfer data directly from the underlying
+ * I/O session's channel to a {@link FileChannel}, whenever
+ * possible avoiding intermediate buffering in the session buffer.
  *
- *
- * @version $Revision: 744538 $
- * 
  * @since 4.0
  */
-public class LengthDelimitedDecoder extends AbstractContentDecoder 
+public class LengthDelimitedDecoder extends AbstractContentDecoder
         implements FileContentDecoder {
-    
+
     private final long contentLength;
-    
+
     private long len;
 
     public LengthDelimitedDecoder(
-            final ReadableByteChannel channel, 
+            final ReadableByteChannel channel,
             final SessionInputBuffer buffer,
             final HttpTransportMetricsImpl metrics,
             long contentLength) {
@@ -82,16 +75,16 @@ public class LengthDelimitedDecoder extends AbstractContentDecoder
         if (this.completed) {
             return -1;
         }
-        int lenRemaining = (int) (this.contentLength - this.len);
-        
+        int chunk = (int) Math.min((this.contentLength - this.len), Integer.MAX_VALUE);
+
         int bytesRead;
         if (this.buffer.hasData()) {
-            int maxLen = Math.min(lenRemaining, this.buffer.length());
+            int maxLen = Math.min(chunk, this.buffer.length());
             bytesRead = this.buffer.read(dst, maxLen);
         } else {
-            if (dst.remaining() > lenRemaining) {
+            if (dst.remaining() > chunk) {
                 int oldLimit = dst.limit();
-                int newLimit = oldLimit - (dst.remaining() - lenRemaining);
+                int newLimit = oldLimit - (dst.remaining() - chunk);
                 dst.limit(newLimit);
                 bytesRead = this.channel.read(dst);
                 dst.limit(oldLimit);
@@ -110,38 +103,42 @@ public class LengthDelimitedDecoder extends AbstractContentDecoder
         if (this.len >= this.contentLength) {
             this.completed = true;
         }
-        return bytesRead;
+        if (this.completed && bytesRead == 0) {
+            return -1;
+        } else {
+            return bytesRead;
+        }
     }
-    
+
     public long transfer(
-            final FileChannel dst, 
-            long position, 
+            final FileChannel dst,
+            long position,
             long count) throws IOException {
-        
+
         if (dst == null) {
             return 0;
         }
         if (this.completed) {
             return -1;
         }
-        
-        int lenRemaining = (int) (this.contentLength - this.len);
-        
+
+        int chunk = (int) Math.min((this.contentLength - this.len), Integer.MAX_VALUE);
+
         long bytesRead;
         if (this.buffer.hasData()) {
-            int maxLen = Math.min(lenRemaining, this.buffer.length());
+            int maxLen = Math.min(chunk, this.buffer.length());
             dst.position(position);
             bytesRead = this.buffer.read(dst, maxLen);
         } else {
-            if (count > lenRemaining) {
-                count = lenRemaining;
+            if (count > chunk) {
+                count = chunk;
             }
             if (this.channel.isOpen()) {
                 if(dst.size() < position)
-                    throw new IOException("FileChannel.size() [" + dst.size() + 
-                                          "] < position [" + position + 
+                    throw new IOException("FileChannel.size() [" + dst.size() +
+                                          "] < position [" + position +
                                           "].  Please grow the file before writing.");
-                
+
                 bytesRead = dst.transferFrom(this.channel, position, count);
             } else {
                 bytesRead = -1;
