@@ -1,8 +1,4 @@
 /*
- * $HeadURL: https://svn.apache.org/repos/asf/httpcomponents/httpcore/tags/4.0.1/httpcore-nio/src/main/java/org/apache/http/nio/protocol/AsyncNHttpServiceHandler.java $
- * $Revision: 747992 $
- * $Date: 2009-02-26 04:03:22 +0100 (Thu, 26 Feb 2009) $
- *
  * ====================================================================
  * Licensed to the Apache Software Foundation (ASF) under one
  * or more contributor license agreements.  See the NOTICE file
@@ -68,34 +64,34 @@ import org.apache.http.protocol.HttpProcessor;
 import org.apache.http.util.EncodingUtils;
 
 /**
- * Fully asynchronous HTTP server side protocol handler implementation that 
- * implements the essential requirements of the HTTP protocol for the server 
- * side message processing as described by RFC 2616. It is capable of processing 
- * HTTP requests with nearly constant memory footprint. Only HTTP message heads 
- * are stored in memory, while content of message bodies is streamed directly 
- * from the entity to the underlying channel (and vice versa) 
+ * Fully asynchronous HTTP server side protocol handler implementation that
+ * implements the essential requirements of the HTTP protocol for the server
+ * side message processing as described by RFC 2616. It is capable of processing
+ * HTTP requests with nearly constant memory footprint. Only HTTP message heads
+ * are stored in memory, while content of message bodies is streamed directly
+ * from the entity to the underlying channel (and vice versa)
  * {@link ConsumingNHttpEntity} and {@link ProducingNHttpEntity} interfaces.
  * <p/>
- * When using this class, it is important to ensure that entities supplied for 
- * writing implement {@link ProducingNHttpEntity}. Doing so will allow the 
- * entity to be written out asynchronously. If entities supplied for writing do 
- * not implement {@link ProducingNHttpEntity}, a delegate is added that buffers 
- * the entire contents in memory. Additionally, the buffering might take place 
- * in the I/O thread, which could cause I/O to block temporarily. For best 
- * results, ensure that all entities set on {@link HttpResponse}s from  
+ * When using this class, it is important to ensure that entities supplied for
+ * writing implement {@link ProducingNHttpEntity}. Doing so will allow the
+ * entity to be written out asynchronously. If entities supplied for writing do
+ * not implement {@link ProducingNHttpEntity}, a delegate is added that buffers
+ * the entire contents in memory. Additionally, the buffering might take place
+ * in the I/O thread, which could cause I/O to block temporarily. For best
+ * results, ensure that all entities set on {@link HttpResponse}s from
  * {@link NHttpRequestHandler}s implement {@link ProducingNHttpEntity}.
  * <p/>
- * If incoming requests enclose a content entity, {@link NHttpRequestHandler}s 
- * are expected to return a {@link ConsumingNHttpEntity} for reading the 
+ * If incoming requests enclose a content entity, {@link NHttpRequestHandler}s
+ * are expected to return a {@link ConsumingNHttpEntity} for reading the
  * content. After the entity is finished reading the data,
  * {@link NHttpRequestHandler#handle(HttpRequest, HttpResponse, NHttpResponseTrigger, HttpContext)}
  * is called to generate a response.
  * <p/>
- * Individual {@link NHttpRequestHandler}s do not have to submit a response 
- * immediately. They can defer transmission of the HTTP response back to the 
- * client without blocking the I/O thread and to delegate the processing the 
- * HTTP request to a worker thread. The worker thread in its turn can use an 
- * instance of {@link NHttpResponseTrigger} passed as a parameter to submit 
+ * Individual {@link NHttpRequestHandler}s do not have to submit a response
+ * immediately. They can defer transmission of the HTTP response back to the
+ * client without blocking the I/O thread and to delegate the processing the
+ * HTTP request to a worker thread. The worker thread in its turn can use an
+ * instance of {@link NHttpResponseTrigger} passed as a parameter to submit
  * a response as at a later point of time once the response becomes available.
  *
  * @see ConsumingNHttpEntity
@@ -177,7 +173,8 @@ public class AsyncNHttpServiceHandler extends NHttpHandlerBase
         try {
 
             if (request instanceof HttpEntityEnclosingRequest) {
-                if (((HttpEntityEnclosingRequest) request).expectContinue()) {
+                HttpEntityEnclosingRequest entityRequest = (HttpEntityEnclosingRequest) request;
+                if (entityRequest.expectContinue()) {
                     response = this.responseFactory.newHttpResponse(
                             ver, HttpStatus.SC_CONTINUE, context);
                     response.setParams(
@@ -207,20 +204,19 @@ public class AsyncNHttpServiceHandler extends NHttpHandlerBase
                     }
                 }
                 // Request content is expected.
-                HttpEntity entity = ((HttpEntityEnclosingRequest) request).getEntity();
+                ConsumingNHttpEntity consumingEntity = null;
 
                 // Lookup request handler for this request
                 if (requestHandler != null) {
-                    ConsumingNHttpEntity consumingEntity = requestHandler.entityRequest(
-                            (HttpEntityEnclosingRequest) request, context);
-                    if (consumingEntity == null) {
-                        consumingEntity = new ConsumingNHttpEntityTemplate(
-                                entity,
-                                new SkipContentListener(this.allocator));
-                    }
-                    ((HttpEntityEnclosingRequest) request).setEntity(consumingEntity);
-                    connState.setConsumingEntity(consumingEntity);
+                    consumingEntity = requestHandler.entityRequest(entityRequest, context);
                 }
+                if (consumingEntity == null) {
+                    consumingEntity = new ConsumingNHttpEntityTemplate(
+                            entityRequest.getEntity(),
+                            new SkipContentListener(this.allocator));
+                }
+                entityRequest.setEntity(consumingEntity);
+                connState.setConsumingEntity(consumingEntity);
 
             } else {
                 // No request content is expected.
@@ -345,12 +341,12 @@ public class AsyncNHttpServiceHandler extends NHttpHandlerBase
 
         try {
 
-            IOException ioex = connState.getIOExepction();
+            IOException ioex = connState.getIOException();
             if (ioex != null) {
                 throw ioex;
             }
 
-            HttpException httpex = connState.getHttpExepction();
+            HttpException httpex = connState.getHttpException();
             if (httpex != null) {
                 HttpResponse response = this.responseFactory.newHttpResponse(HttpVersion.HTTP_1_0,
                         HttpStatus.SC_INTERNAL_SERVER_ERROR, context);
@@ -427,7 +423,7 @@ public class AsyncNHttpServiceHandler extends NHttpHandlerBase
     }
 
     /**
-     * @throws HttpException - not thrown currently 
+     * @throws HttpException - not thrown currently
      */
     private void processRequest(
             final NHttpServerConnection conn,
@@ -609,18 +605,38 @@ public class AsyncNHttpServiceHandler extends NHttpHandlerBase
             this.producingEntity = producingEntity;
         }
 
+        public IOException getIOException() {
+            return this.ioex;
+        }
+
+        @Deprecated
         public IOException getIOExepction() {
             return this.ioex;
         }
 
+        public void setIOException(final IOException ex) {
+            this.ioex = ex;
+        }
+
+        @Deprecated
         public void setIOExepction(final IOException ex) {
             this.ioex = ex;
         }
 
+        public HttpException getHttpException() {
+            return this.httpex;
+        }
+
+        @Deprecated
         public HttpException getHttpExepction() {
             return this.httpex;
         }
 
+        public void setHttpException(final HttpException ex) {
+            this.httpex = ex;
+        }
+
+        @Deprecated
         public void setHttpExepction(final HttpException ex) {
             this.httpex = ex;
         }
@@ -665,7 +681,7 @@ public class AsyncNHttpServiceHandler extends NHttpHandlerBase
                 throw new IllegalStateException("Response already triggered");
             }
             this.triggered = true;
-            this.connState.setHttpExepction(ex);
+            this.connState.setHttpException(ex);
             this.iocontrol.requestOutput();
         }
 
@@ -674,7 +690,7 @@ public class AsyncNHttpServiceHandler extends NHttpHandlerBase
                 throw new IllegalStateException("Response already triggered");
             }
             this.triggered = true;
-            this.connState.setIOExepction(ex);
+            this.connState.setIOException(ex);
             this.iocontrol.requestOutput();
         }
 
