@@ -1,8 +1,4 @@
 /*
- * $HeadURL: https://svn.apache.org/repos/asf/httpcomponents/httpcore/tags/4.0.1/httpcore-nio/src/main/java/org/apache/http/nio/util/BufferInfo.java $
- * $Revision: 744545 $
- * $Date: 2009-02-14 18:34:58 +0100 (Sat, 14 Feb 2009) $
- *
  * ====================================================================
  * Licensed to the Apache Software Foundation (ASF) under one
  * or more contributor license agreements.  See the NOTICE file
@@ -33,16 +29,12 @@ package org.apache.http.nio.util;
 
 /**
  * Basic buffer properties.
- * <p>
- * This interface should be eliminated in the next major release and its 
- * methods added to {@link org.apache.http.nio.reactor.SessionOutputBuffer}
- * and {@link org.apache.http.nio.reactor.SessionInputBuffer} interfaces.
- *
- *
- * @version $Revision: 744545 $
  *
  * @since 4.0
+ *
+ * @deprecated Use {@link org.apache.http.io.BufferInfo}
  */
+@Deprecated
 public interface BufferInfo {
 
     int length();
