@@ -1,7 +1,4 @@
 /*
- * $HeadURL: https://svn.apache.org/repos/asf/httpcomponents/httpcore/tags/4.0.1/httpcore-nio/src/test/java/org/apache/http/impl/nio/codecs/TestLengthDelimitedDecoder.java $
- * $Revision: 744515 $
- * $Date: 2009-02-14 17:36:56 +0100 (Sat, 14 Feb 2009) $
  * ====================================================================
  * Licensed to the Apache Software Foundation (ASF) under one
  * or more contributor license agreements.  See the NOTICE file
@@ -39,9 +36,7 @@ import java.nio.ByteBuffer;
 import java.nio.channels.FileChannel;
 import java.nio.channels.ReadableByteChannel;
 
-import junit.framework.Test;
 import junit.framework.TestCase;
-import junit.framework.TestSuite;
 
 import org.apache.http.impl.io.HttpTransportMetricsImpl;
 import org.apache.http.impl.nio.reactor.SessionInputBufferImpl;
@@ -53,8 +48,8 @@ import org.apache.http.params.HttpParams;
 /**
  * Simple tests for {@link LengthDelimitedDecoder}.
  *
- * 
- * @version $Id: TestLengthDelimitedDecoder.java 744515 2009-02-14 16:36:56Z sebb $
+ *
+ * @version $Id: TestLengthDelimitedDecoder.java 937477 2010-04-23 19:11:53Z olegk $
  */
 public class TestLengthDelimitedDecoder extends TestCase {
 
@@ -63,27 +58,17 @@ public class TestLengthDelimitedDecoder extends TestCase {
         super(testName);
     }
 
-    // ------------------------------------------------------------------- Main
-    public static void main(String args[]) {
-        String[] testCaseName = { TestLengthDelimitedDecoder.class.getName() };
-        junit.textui.TestRunner.main(testCaseName);
-    }
-
     // ------------------------------------------------------- TestCase Methods
 
-    public static Test suite() {
-        return new TestSuite(TestLengthDelimitedDecoder.class);
-    }
-
     private static String convert(final ByteBuffer src) {
         src.flip();
-        StringBuffer buffer = new StringBuffer(src.remaining()); 
+        StringBuffer buffer = new StringBuffer(src.remaining());
         while (src.hasRemaining()) {
             buffer.append((char)(src.get() & 0xff));
         }
         return buffer.toString();
     }
-    
+
     private static String readFromFile(final File file) throws Exception {
         FileInputStream filestream = new FileInputStream(file);
         InputStreamReader reader = new InputStreamReader(filestream);
@@ -102,63 +87,63 @@ public class TestLengthDelimitedDecoder extends TestCase {
 
     public void testBasicDecoding() throws Exception {
         ReadableByteChannel channel = new ReadableByteChannelMockup(
-                new String[] {"stuff;", "more stuff"}, "US-ASCII"); 
+                new String[] {"stuff;", "more stuff"}, "US-ASCII");
         HttpParams params = new BasicHttpParams();
-        
-        SessionInputBuffer inbuf = new SessionInputBufferImpl(1024, 256, params); 
+
+        SessionInputBuffer inbuf = new SessionInputBufferImpl(1024, 256, params);
         HttpTransportMetricsImpl metrics = new HttpTransportMetricsImpl();
         LengthDelimitedDecoder decoder = new LengthDelimitedDecoder(
-                channel, inbuf, metrics, 16); 
-        
-        ByteBuffer dst = ByteBuffer.allocate(1024); 
-        
+                channel, inbuf, metrics, 16);
+
+        ByteBuffer dst = ByteBuffer.allocate(1024);
+
         int bytesRead = decoder.read(dst);
         assertEquals(6, bytesRead);
         assertEquals("stuff;", convert(dst));
         assertFalse(decoder.isCompleted());
         assertEquals(6, metrics.getBytesTransferred());
-        
+
         dst.clear();
         bytesRead = decoder.read(dst);
         assertEquals(10, bytesRead);
         assertEquals("more stuff", convert(dst));
         assertTrue(decoder.isCompleted());
         assertEquals(16, metrics.getBytesTransferred());
-        
+
         dst.clear();
         bytesRead = decoder.read(dst);
         assertEquals(-1, bytesRead);
         assertTrue(decoder.isCompleted());
         assertEquals(16, metrics.getBytesTransferred());
     }
-    
+
     public void testCodingBeyondContentLimit() throws Exception {
         ReadableByteChannel channel = new ReadableByteChannelMockup(
                 new String[] {
-                        "stuff;", 
-                        "more stuff; and a lot more stuff"}, "US-ASCII"); 
+                        "stuff;",
+                        "more stuff; and a lot more stuff"}, "US-ASCII");
         HttpParams params = new BasicHttpParams();
-        
-        SessionInputBuffer inbuf = new SessionInputBufferImpl(1024, 256, params); 
+
+        SessionInputBuffer inbuf = new SessionInputBufferImpl(1024, 256, params);
         HttpTransportMetricsImpl metrics = new HttpTransportMetricsImpl();
         LengthDelimitedDecoder decoder = new LengthDelimitedDecoder(
-                channel, inbuf, metrics, 16); 
-        
-        ByteBuffer dst = ByteBuffer.allocate(1024); 
-        
+                channel, inbuf, metrics, 16);
+
+        ByteBuffer dst = ByteBuffer.allocate(1024);
+
         int bytesRead = decoder.read(dst);
         assertEquals(6, bytesRead);
         assertEquals("stuff;", convert(dst));
         assertFalse(decoder.isCompleted());
         assertEquals(6, metrics.getBytesTransferred());
-        
+
         dst.clear();
         bytesRead = decoder.read(dst);
         assertEquals(10, bytesRead);
         assertEquals("more stuff", convert(dst));
         assertTrue(decoder.isCompleted());
         assertEquals(16, metrics.getBytesTransferred());
-        
+
         dst.clear();
         bytesRead = decoder.read(dst);
         assertEquals(-1, bytesRead);
@@ -168,29 +153,29 @@ public class TestLengthDelimitedDecoder extends TestCase {
 
     public void testBasicDecodingSmallBuffer() throws Exception {
         ReadableByteChannel channel = new ReadableByteChannelMockup(
-                new String[] {"stuff;", "more stuff"}, "US-ASCII"); 
+                new String[] {"stuff;", "more stuff"}, "US-ASCII");
         HttpParams params = new BasicHttpParams();
-        
-        SessionInputBuffer inbuf = new SessionInputBufferImpl(1024, 256, params); 
+
+        SessionInputBuffer inbuf = new SessionInputBufferImpl(1024, 256, params);
         HttpTransportMetricsImpl metrics = new HttpTransportMetricsImpl();
         LengthDelimitedDecoder decoder = new LengthDelimitedDecoder(
-                channel, inbuf, metrics, 16); 
-        
-        ByteBuffer dst = ByteBuffer.allocate(4); 
-        
+                channel, inbuf, metrics, 16);
+
+        ByteBuffer dst = ByteBuffer.allocate(4);
+
         int bytesRead = decoder.read(dst);
         assertEquals(4, bytesRead);
         assertEquals("stuf", convert(dst));
         assertFalse(decoder.isCompleted());
         assertEquals(4, metrics.getBytesTransferred());
-        
+
         dst.clear();
         bytesRead = decoder.read(dst);
         assertEquals(2, bytesRead);
         assertEquals("f;", convert(dst));
         assertFalse(decoder.isCompleted());
         assertEquals(6, metrics.getBytesTransferred());
-        
+
         dst.clear();
         bytesRead = decoder.read(dst);
         assertEquals(4, bytesRead);
@@ -218,37 +203,37 @@ public class TestLengthDelimitedDecoder extends TestCase {
         assertTrue(decoder.isCompleted());
         assertEquals(16, metrics.getBytesTransferred());
     }
-    
+
     public void testDecodingFromSessionBuffer1() throws Exception {
         ReadableByteChannel channel = new ReadableByteChannelMockup(
-                new String[] {"stuff;", "more stuff"}, "US-ASCII"); 
+                new String[] {"stuff;", "more stuff"}, "US-ASCII");
         HttpParams params = new BasicHttpParams();
-        
+
         SessionInputBuffer inbuf = new SessionInputBufferImpl(1024, 256, params);
         HttpTransportMetricsImpl metrics = new HttpTransportMetricsImpl();
 
         inbuf.fill(channel);
-        
+
         assertEquals(6, inbuf.length());
-        
+
         LengthDelimitedDecoder decoder = new LengthDelimitedDecoder(
-                channel, inbuf, metrics, 16); 
-        
-        ByteBuffer dst = ByteBuffer.allocate(1024); 
-        
+                channel, inbuf, metrics, 16);
+
+        ByteBuffer dst = ByteBuffer.allocate(1024);
+
         int bytesRead = decoder.read(dst);
         assertEquals(6, bytesRead);
         assertEquals("stuff;", convert(dst));
         assertFalse(decoder.isCompleted());
         assertEquals(0, metrics.getBytesTransferred());
-        
+
         dst.clear();
         bytesRead = decoder.read(dst);
         assertEquals(10, bytesRead);
         assertEquals("more stuff", convert(dst));
         assertTrue(decoder.isCompleted());
         assertEquals(10, metrics.getBytesTransferred());
-        
+
         dst.clear();
         bytesRead = decoder.read(dst);
         assertEquals(-1, bytesRead);
@@ -259,29 +244,29 @@ public class TestLengthDelimitedDecoder extends TestCase {
     public void testDecodingFromSessionBuffer2() throws Exception {
         ReadableByteChannel channel = new ReadableByteChannelMockup(
                 new String[] {
-                        "stuff;", 
-                        "more stuff; and a lot more stuff"}, "US-ASCII"); 
+                        "stuff;",
+                        "more stuff; and a lot more stuff"}, "US-ASCII");
         HttpParams params = new BasicHttpParams();
-        
+
         SessionInputBuffer inbuf = new SessionInputBufferImpl(1024, 256, params);
         HttpTransportMetricsImpl metrics = new HttpTransportMetricsImpl();
 
         inbuf.fill(channel);
         inbuf.fill(channel);
-        
+
         assertEquals(38, inbuf.length());
-        
+
         LengthDelimitedDecoder decoder = new LengthDelimitedDecoder(
-                channel, inbuf, metrics, 16); 
-        
-        ByteBuffer dst = ByteBuffer.allocate(1024); 
-        
+                channel, inbuf, metrics, 16);
+
+        ByteBuffer dst = ByteBuffer.allocate(1024);
+
         int bytesRead = decoder.read(dst);
         assertEquals(16, bytesRead);
         assertEquals("stuff;more stuff", convert(dst));
         assertTrue(decoder.isCompleted());
         assertEquals(0, metrics.getBytesTransferred());
-        
+
         dst.clear();
         bytesRead = decoder.read(dst);
         assertEquals(-1, bytesRead);
@@ -291,19 +276,19 @@ public class TestLengthDelimitedDecoder extends TestCase {
 
     public void testBasicDecodingFile() throws Exception {
         ReadableByteChannel channel = new ReadableByteChannelMockup(
-                new String[] {"stuff; ", "more stuff; ", "a lot more stuff!!!"}, "US-ASCII"); 
+                new String[] {"stuff; ", "more stuff; ", "a lot more stuff!!!"}, "US-ASCII");
         HttpParams params = new BasicHttpParams();
-        
-        SessionInputBuffer inbuf = new SessionInputBufferImpl(1024, 256, params); 
+
+        SessionInputBuffer inbuf = new SessionInputBufferImpl(1024, 256, params);
         HttpTransportMetricsImpl metrics = new HttpTransportMetricsImpl();
         LengthDelimitedDecoder decoder = new LengthDelimitedDecoder(
-                channel, inbuf, metrics, 36); 
-        
+                channel, inbuf, metrics, 36);
+
         File fileHandle = File.createTempFile("testFile", ".txt");
 
-        RandomAccessFile testfile = new RandomAccessFile(fileHandle, "rw"); 
+        RandomAccessFile testfile = new RandomAccessFile(fileHandle, "rw");
         FileChannel fchannel = testfile.getChannel();
-            
+
         long pos = 0;
         while (!decoder.isCompleted()) {
             long bytesRead = decoder.transfer(fchannel, pos, 10);
@@ -313,30 +298,36 @@ public class TestLengthDelimitedDecoder extends TestCase {
         }
         assertEquals(testfile.length(), metrics.getBytesTransferred());
         fchannel.close();
-        
+
         assertEquals("stuff; more stuff; a lot more stuff!", readFromFile(fileHandle));
-        
-        fileHandle.delete();
+
+        deleteWithCheck(fileHandle);
     }
-    
+
+    private void deleteWithCheck(File handle){
+        if (!handle.delete() && handle.exists()){
+            System.err.println("Failed to delete: "+handle.getPath());
+        }
+    }
+
     public void testDecodingFileWithBufferedSessionData() throws Exception {
         ReadableByteChannel channel = new ReadableByteChannelMockup(
-                new String[] {"stuff; ", "more stuff; ", "a lot more stuff!!!"}, "US-ASCII"); 
+                new String[] {"stuff; ", "more stuff; ", "a lot more stuff!!!"}, "US-ASCII");
         HttpParams params = new BasicHttpParams();
-        
-        SessionInputBuffer inbuf = new SessionInputBufferImpl(1024, 256, params); 
+
+        SessionInputBuffer inbuf = new SessionInputBufferImpl(1024, 256, params);
         HttpTransportMetricsImpl metrics = new HttpTransportMetricsImpl();
         LengthDelimitedDecoder decoder = new LengthDelimitedDecoder(
-                channel, inbuf, metrics, 36); 
-        
+                channel, inbuf, metrics, 36);
+
         int i = inbuf.fill(channel);
         assertEquals(7, i);
-        
+
         File fileHandle = File.createTempFile("testFile", ".txt");
 
-        RandomAccessFile testfile = new RandomAccessFile(fileHandle, "rw"); 
+        RandomAccessFile testfile = new RandomAccessFile(fileHandle, "rw");
         FileChannel fchannel = testfile.getChannel();
-            
+
         long pos = 0;
         while (!decoder.isCompleted()) {
             long bytesRead = decoder.transfer(fchannel, pos, 10);
@@ -344,38 +335,38 @@ public class TestLengthDelimitedDecoder extends TestCase {
                 pos += bytesRead;
             }
         }
-        
+
         assertEquals(testfile.length() - 7, metrics.getBytesTransferred());
         fchannel.close();
-        
+
         assertEquals("stuff; more stuff; a lot more stuff!", readFromFile(fileHandle));
-        
-        fileHandle.delete();
+
+        deleteWithCheck(fileHandle);
     }
-    
+
     public void testDecodingFileWithOffsetAndBufferedSessionData() throws Exception {
         ReadableByteChannel channel = new ReadableByteChannelMockup(
-                new String[] {"stuff; ", "more stuff; ", "a lot more stuff!"}, "US-ASCII"); 
+                new String[] {"stuff; ", "more stuff; ", "a lot more stuff!"}, "US-ASCII");
         HttpParams params = new BasicHttpParams();
-        
-        SessionInputBuffer inbuf = new SessionInputBufferImpl(1024, 256, params); 
+
+        SessionInputBuffer inbuf = new SessionInputBufferImpl(1024, 256, params);
         HttpTransportMetricsImpl metrics = new HttpTransportMetricsImpl();
         LengthDelimitedDecoder decoder = new LengthDelimitedDecoder(
-                channel, inbuf, metrics, 36); 
-        
+                channel, inbuf, metrics, 36);
+
         int i = inbuf.fill(channel);
         assertEquals(7, i);
-        
+
         File fileHandle = File.createTempFile("testFile", ".txt");
 
         RandomAccessFile testfile = new RandomAccessFile(fileHandle, "rw");
         byte[] beginning = "beginning; ".getBytes("US-ASCII");
         testfile.write(beginning);
         testfile.close();
-        
+
         testfile = new RandomAccessFile(fileHandle, "rw");
         FileChannel fchannel = testfile.getChannel();
-            
+
         long pos = beginning.length;
         while (!decoder.isCompleted()) {
             if(testfile.length() < pos)
@@ -385,80 +376,82 @@ public class TestLengthDelimitedDecoder extends TestCase {
                 pos += bytesRead;
             }
         }
-        
+
         // count everything except the initial 7 bytes that went to the session buffer
         assertEquals(testfile.length() - 7 - beginning.length, metrics.getBytesTransferred());
         fchannel.close();
-        
+
         assertEquals("beginning; stuff; more stuff; a lot more stuff!", readFromFile(fileHandle));
-        
-        fileHandle.delete();
+
+        deleteWithCheck(fileHandle);
     }
-    
+
     public void testWriteBeyondFileSize() throws Exception {
         ReadableByteChannel channel = new ReadableByteChannelMockup(
-                new String[] {"a"}, "US-ASCII"); 
+                new String[] {"a"}, "US-ASCII");
         HttpParams params = new BasicHttpParams();
-        
-        SessionInputBuffer inbuf = new SessionInputBufferImpl(1024, 256, params); 
+
+        SessionInputBuffer inbuf = new SessionInputBufferImpl(1024, 256, params);
         HttpTransportMetricsImpl metrics = new HttpTransportMetricsImpl();
         LengthDelimitedDecoder decoder = new LengthDelimitedDecoder(
-                channel, inbuf, metrics, 1); 
-        
+                channel, inbuf, metrics, 1);
+
         File fileHandle = File.createTempFile("testFile", ".txt");
 
         RandomAccessFile testfile = new RandomAccessFile(fileHandle, "rw");
         FileChannel fchannel = testfile.getChannel();
         assertEquals(0, testfile.length());
-            
+
         try {
             decoder.transfer(fchannel, 5, 10);
             fail("expected IOException");
         } catch(IOException iox) {}
-        
-        fileHandle.delete();
+
+        testfile.close();
+        deleteWithCheck(fileHandle);
     }
-    
+
     public void testCodingBeyondContentLimitFile() throws Exception {
         ReadableByteChannel channel = new ReadableByteChannelMockup(
                 new String[] {
-                        "stuff;", 
-                        "more stuff; and a lot more stuff"}, "US-ASCII"); 
+                        "stuff;",
+                        "more stuff; and a lot more stuff"}, "US-ASCII");
         HttpParams params = new BasicHttpParams();
-        
-        SessionInputBuffer inbuf = new SessionInputBufferImpl(1024, 256, params); 
+
+        SessionInputBuffer inbuf = new SessionInputBufferImpl(1024, 256, params);
         HttpTransportMetricsImpl metrics = new HttpTransportMetricsImpl();
         LengthDelimitedDecoder decoder = new LengthDelimitedDecoder(
-                channel, inbuf, metrics, 16); 
-        
+                channel, inbuf, metrics, 16);
+
         File fileHandle = File.createTempFile("testFile", ".txt");
         RandomAccessFile testfile  = new RandomAccessFile(fileHandle, "rw");
         FileChannel fchannel = testfile.getChannel();
-        
+
         long bytesRead = decoder.transfer(fchannel, 0, 6);
         assertEquals(6, bytesRead);
         assertFalse(decoder.isCompleted());
         assertEquals(6, metrics.getBytesTransferred());
-        
+
         bytesRead = decoder.transfer(fchannel,0 , 10);
         assertEquals(10, bytesRead);
         assertTrue(decoder.isCompleted());
         assertEquals(16, metrics.getBytesTransferred());
-        
+
         bytesRead = decoder.transfer(fchannel, 0, 1);
         assertEquals(-1, bytesRead);
         assertTrue(decoder.isCompleted());
         assertEquals(16, metrics.getBytesTransferred());
-        
-        fileHandle.delete();
+
+        testfile.close();
+        deleteWithCheck(fileHandle);
     }
-    
+
     public void testInvalidConstructor() {
         ReadableByteChannel channel = new ReadableByteChannelMockup(
-                new String[] {"stuff;", "more stuff"}, "US-ASCII"); 
+                new String[] {"stuff;", "more stuff"}, "US-ASCII");
         HttpParams params = new BasicHttpParams();
-        
-        SessionInputBuffer inbuf = new SessionInputBufferImpl(1024, 256, params); 
+
+        SessionInputBuffer inbuf = new SessionInputBufferImpl(1024, 256, params);
         HttpTransportMetricsImpl metrics = new HttpTransportMetricsImpl();
         try {
             new LengthDelimitedDecoder(null, null, null, 10);
@@ -489,14 +482,14 @@ public class TestLengthDelimitedDecoder extends TestCase {
     public void testInvalidInput() throws Exception {
         String s = "stuff";
         ReadableByteChannel channel = new ReadableByteChannelMockup(
-                new String[] {s}, "US-ASCII"); 
+                new String[] {s}, "US-ASCII");
         HttpParams params = new BasicHttpParams();
-    
-        SessionInputBuffer inbuf = new SessionInputBufferImpl(1024, 256, params); 
+
+        SessionInputBuffer inbuf = new SessionInputBufferImpl(1024, 256, params);
         HttpTransportMetricsImpl metrics = new HttpTransportMetricsImpl();
         LengthDelimitedDecoder decoder = new LengthDelimitedDecoder(
                 channel, inbuf, metrics, 3);
-        
+
         try {
             decoder.read(null);
             fail("IllegalArgumentException should have been thrown");
@@ -504,5 +497,23 @@ public class TestLengthDelimitedDecoder extends TestCase {
             // expected
         }
     }
-    
+
+    public void testZeroLengthDecoding() throws Exception {
+        ReadableByteChannel channel = new ReadableByteChannelMockup(
+                new String[] {"stuff"}, "US-ASCII");
+        HttpParams params = new BasicHttpParams();
+
+        SessionInputBuffer inbuf = new SessionInputBufferImpl(1024, 256, params);
+        HttpTransportMetricsImpl metrics = new HttpTransportMetricsImpl();
+        LengthDelimitedDecoder decoder = new LengthDelimitedDecoder(
+                channel, inbuf, metrics, 0);
+
+        ByteBuffer dst = ByteBuffer.allocate(1024);
+
+        int bytesRead = decoder.read(dst);
+        assertEquals(-1, bytesRead);
+        assertTrue(decoder.isCompleted());
+        assertEquals(0, metrics.getBytesTransferred());
+    }
+
 }
