@@ -1,7 +1,4 @@
 /*
- * $HeadURL: https://svn.apache.org/repos/asf/httpcomponents/httpcore/tags/4.0.1/httpcore-nio/src/test/java/org/apache/http/impl/nio/reactor/TestDefaultIOReactors.java $
- * $Revision: 744515 $
- * $Date: 2009-02-14 17:36:56 +0100 (Sat, 14 Feb 2009) $
  * ====================================================================
  * Licensed to the Apache Software Foundation (ASF) under one
  * or more contributor license agreements.  See the NOTICE file
@@ -32,35 +29,50 @@ package org.apache.http.impl.nio.reactor;
 
 import java.io.IOException;
 import java.net.InetSocketAddress;
+import java.util.LinkedList;
 import java.util.List;
-
-import junit.framework.Test;
-import junit.framework.TestSuite;
+import java.util.Queue;
+import java.util.concurrent.CountDownLatch;
+import java.util.concurrent.atomic.AtomicInteger;
 
 import org.apache.http.HttpCoreNIOTestBase;
 import org.apache.http.HttpException;
 import org.apache.http.HttpRequest;
+import org.apache.http.HttpRequestInterceptor;
 import org.apache.http.HttpResponse;
+import org.apache.http.HttpResponseInterceptor;
+import org.apache.http.OoopsieRuntimeException;
+import org.apache.http.impl.DefaultConnectionReuseStrategy;
+import org.apache.http.impl.DefaultHttpResponseFactory;
 import org.apache.http.message.BasicHttpRequest;
-import org.apache.http.mockup.RequestCount;
 import org.apache.http.mockup.SimpleEventListener;
-import org.apache.http.nio.NHttpClientHandler;
+import org.apache.http.mockup.SimpleHttpRequestHandlerResolver;
 import org.apache.http.nio.NHttpConnection;
-import org.apache.http.nio.NHttpServiceHandler;
+import org.apache.http.nio.protocol.BufferingHttpClientHandler;
+import org.apache.http.nio.protocol.BufferingHttpServiceHandler;
 import org.apache.http.nio.protocol.EventListener;
 import org.apache.http.nio.protocol.HttpRequestExecutionHandler;
 import org.apache.http.nio.reactor.IOReactorException;
 import org.apache.http.nio.reactor.IOReactorExceptionHandler;
 import org.apache.http.nio.reactor.IOReactorStatus;
 import org.apache.http.nio.reactor.ListenerEndpoint;
+import org.apache.http.nio.reactor.SessionRequest;
 import org.apache.http.protocol.HttpContext;
+import org.apache.http.protocol.HttpProcessor;
 import org.apache.http.protocol.HttpRequestHandler;
+import org.apache.http.protocol.ImmutableHttpProcessor;
+import org.apache.http.protocol.RequestConnControl;
+import org.apache.http.protocol.RequestContent;
+import org.apache.http.protocol.RequestExpectContinue;
+import org.apache.http.protocol.RequestTargetHost;
+import org.apache.http.protocol.RequestUserAgent;
+import org.apache.http.protocol.ResponseConnControl;
+import org.apache.http.protocol.ResponseContent;
+import org.apache.http.protocol.ResponseDate;
+import org.apache.http.protocol.ResponseServer;
 
 /**
  * Tests for basic I/O functionality.
- *
- * 
- * @version $Id: TestDefaultIOReactors.java 744515 2009-02-14 16:36:56Z sebb $
  */
 public class TestDefaultIOReactors extends HttpCoreNIOTestBase {
 
@@ -69,44 +81,39 @@ public class TestDefaultIOReactors extends HttpCoreNIOTestBase {
         super(testName);
     }
 
-    // ------------------------------------------------------------------- Main
-    public static void main(String args[]) {
-        String[] testCaseName = { TestDefaultIOReactors.class.getName() };
-        junit.textui.TestRunner.main(testCaseName);
-    }
-
     // ------------------------------------------------------- TestCase Methods
 
-    public static Test suite() {
-        return new TestSuite(TestDefaultIOReactors.class);
-    }
-
     public void testGracefulShutdown() throws Exception {
 
-        // Open some connection and make sure 
+        // Open some connection and make sure
         // they get cleanly closed upon shutdown
-        
+
         final int connNo = 10;
-        final RequestCount requestConns = new RequestCount(connNo); 
-        final RequestCount closedServerConns = new RequestCount(connNo); 
-        final RequestCount closedClientConns = new RequestCount(connNo); 
-        
+        final CountDownLatch requestConns = new CountDownLatch(connNo);
+        final AtomicInteger closedServerConns = new AtomicInteger(0);
+        final AtomicInteger openServerConns = new AtomicInteger(0);
+        final AtomicInteger closedClientConns = new AtomicInteger(0);
+        final AtomicInteger openClientConns = new AtomicInteger(0);
+
         HttpRequestHandler requestHandler = new HttpRequestHandler() {
 
             public void handle(
-                    final HttpRequest request, 
-                    final HttpResponse response, 
+                    final HttpRequest request,
+                    final HttpResponse response,
                     final HttpContext context) throws HttpException, IOException {
             }
-            
+
         };
-        
+
         HttpRequestExecutionHandler requestExecutionHandler = new HttpRequestExecutionHandler() {
 
             public void initalizeContext(final HttpContext context, final Object attachment) {
             }
 
             public void finalizeContext(final HttpContext context) {
+                while (requestConns.getCount() > 0) {
+                    requestConns.countDown();
+                }
             }
 
             public HttpRequest submitRequest(final HttpContext context) {
@@ -119,82 +126,130 @@ public class TestDefaultIOReactors extends HttpCoreNIOTestBase {
                     return null;
                 }
             }
-            
+
             public void handleResponse(final HttpResponse response, final HttpContext context) {
-                requestConns.decrement();                    
+                requestConns.countDown();
             }
-            
+
         };
-     
+
         EventListener serverEventListener = new SimpleEventListener() {
 
             @Override
-            public void connectionClosed(NHttpConnection conn) {
-                closedServerConns.decrement();
-                super.connectionClosed(conn);
+            public void connectionOpen(NHttpConnection conn) {
+                openServerConns.incrementAndGet();
+                super.connectionOpen(conn);
             }
-            
-        };
-        
-        NHttpServiceHandler serviceHandler = createHttpServiceHandler(
-                requestHandler, 
-                null,
-                serverEventListener);
-        
-        EventListener clientEventListener = new SimpleEventListener() {
 
             @Override
             public void connectionClosed(NHttpConnection conn) {
-                closedClientConns.decrement();
+                closedServerConns.incrementAndGet();
                 super.connectionClosed(conn);
             }
-            
+
         };
-        
-        NHttpClientHandler clientHandler = createHttpClientHandler(
+
+        HttpProcessor serverHttpProc = new ImmutableHttpProcessor(new HttpResponseInterceptor[] {
+                new ResponseDate(),
+                new ResponseServer(),
+                new ResponseContent(),
+                new ResponseConnControl()
+        });
+
+        BufferingHttpServiceHandler serviceHandler = new BufferingHttpServiceHandler(
+                serverHttpProc,
+                new DefaultHttpResponseFactory(),
+                new DefaultConnectionReuseStrategy(),
+                this.server.getParams());
+
+        serviceHandler.setHandlerResolver(
+                new SimpleHttpRequestHandlerResolver(requestHandler));
+        serviceHandler.setEventListener(
+                serverEventListener);
+
+        EventListener clientEventListener = new SimpleEventListener() {
+
+            @Override
+            public void connectionOpen(NHttpConnection conn) {
+                openClientConns.incrementAndGet();
+                super.connectionOpen(conn);
+            }
+
+            @Override
+            public void connectionClosed(NHttpConnection conn) {
+                closedClientConns.incrementAndGet();
+                super.connectionClosed(conn);
+            }
+
+        };
+
+        HttpProcessor clientHttpProc = new ImmutableHttpProcessor(new HttpRequestInterceptor[] {
+                new RequestContent(),
+                new RequestTargetHost(),
+                new RequestConnControl(),
+                new RequestUserAgent(),
+                new RequestExpectContinue()});
+
+        BufferingHttpClientHandler clientHandler = new BufferingHttpClientHandler(
+                clientHttpProc,
                 requestExecutionHandler,
+                new DefaultConnectionReuseStrategy(),
+                this.client.getParams());
+
+        clientHandler.setEventListener(
                 clientEventListener);
 
         this.server.start(serviceHandler);
         this.client.start(clientHandler);
-        
-        
+
         ListenerEndpoint endpoint = this.server.getListenerEndpoint();
         endpoint.waitFor();
         InetSocketAddress serverAddress = (InetSocketAddress) endpoint.getAddress();
-        
+
+        assertEquals("Test server status", IOReactorStatus.ACTIVE, this.server.getStatus());
+
+        Queue<SessionRequest> connRequests = new LinkedList<SessionRequest>();
         for (int i = 0; i < connNo; i++) {
-            this.client.openConnection(
-                    new InetSocketAddress("localhost", serverAddress.getPort()), 
+            SessionRequest sessionRequest = this.client.openConnection(
+                    new InetSocketAddress("localhost", serverAddress.getPort()),
                     null);
+            connRequests.add(sessionRequest);
         }
-     
-        requestConns.await(10000);
-        assertEquals(0, requestConns.getValue());
-     
+
+        while (!connRequests.isEmpty()) {
+            SessionRequest sessionRequest = connRequests.remove();
+            sessionRequest.waitFor();
+            if (sessionRequest.getException() != null) {
+                throw sessionRequest.getException();
+            }
+            assertNotNull(sessionRequest.getSession());
+        }
+
+        assertEquals("Test client status", IOReactorStatus.ACTIVE, this.client.getStatus());
+
+        requestConns.await();
+        assertEquals(0, requestConns.getCount());
+
         this.client.shutdown();
         this.server.shutdown();
-        
-        closedClientConns.await(10000);
-        assertEquals(0, closedClientConns.getValue());
-     
-        closedServerConns.await(10000);
-        assertEquals(0, closedServerConns.getValue());
+
+        assertEquals(openClientConns.get(), closedClientConns.get());
+        assertEquals(openServerConns.get(), closedServerConns.get());
     }
-    
+
     public void testRuntimeException() throws Exception {
 
         HttpRequestHandler requestHandler = new HttpRequestHandler() {
 
             public void handle(
-                    final HttpRequest request, 
-                    final HttpResponse response, 
+                    final HttpRequest request,
+                    final HttpResponse response,
                     final HttpContext context) throws HttpException, IOException {
-                throw new IllegalStateException("Oppsie!!!");
+                throw new OoopsieRuntimeException();
             }
-            
+
         };
-        
+
         HttpRequestExecutionHandler requestExecutionHandler = new HttpRequestExecutionHandler() {
 
             public void initalizeContext(final HttpContext context, final Object attachment) {
@@ -213,66 +268,91 @@ public class TestDefaultIOReactors extends HttpCoreNIOTestBase {
                     return null;
                 }
             }
-            
+
             public void handleResponse(final HttpResponse response, final HttpContext context) {
             }
-            
+
         };
-     
-        NHttpServiceHandler serviceHandler = createHttpServiceHandler(
-                requestHandler, 
-                null,
+
+        HttpProcessor serverHttpProc = new ImmutableHttpProcessor(new HttpResponseInterceptor[] {
+                new ResponseDate(),
+                new ResponseServer(),
+                new ResponseContent(),
+                new ResponseConnControl()
+        });
+
+        BufferingHttpServiceHandler serviceHandler = new BufferingHttpServiceHandler(
+                serverHttpProc,
+                new DefaultHttpResponseFactory(),
+                new DefaultConnectionReuseStrategy(),
+                this.server.getParams());
+
+        serviceHandler.setHandlerResolver(
+                new SimpleHttpRequestHandlerResolver(requestHandler));
+        serviceHandler.setEventListener(
                 new SimpleEventListener());
-        
-        NHttpClientHandler clientHandler = createHttpClientHandler(
+
+        HttpProcessor clientHttpProc = new ImmutableHttpProcessor(new HttpRequestInterceptor[] {
+                new RequestContent(),
+                new RequestTargetHost(),
+                new RequestConnControl(),
+                new RequestUserAgent(),
+                new RequestExpectContinue()});
+
+        BufferingHttpClientHandler clientHandler = new BufferingHttpClientHandler(
+                clientHttpProc,
                 requestExecutionHandler,
+                new DefaultConnectionReuseStrategy(),
+                this.client.getParams());
+
+        clientHandler.setEventListener(
                 new SimpleEventListener());
 
         this.server.start(serviceHandler);
         this.client.start(clientHandler);
-        
+
         ListenerEndpoint endpoint = this.server.getListenerEndpoint();
         endpoint.waitFor();
         InetSocketAddress serverAddress = (InetSocketAddress) endpoint.getAddress();
-        
+
         this.client.openConnection(
-                new InetSocketAddress("localhost", serverAddress.getPort()), 
+                new InetSocketAddress("localhost", serverAddress.getPort()),
                 null);
-     
+
         this.server.join(20000);
-        
+
         Exception ex = this.server.getException();
         assertNotNull(ex);
         assertTrue(ex instanceof IOReactorException);
         assertNotNull(ex.getCause());
-        assertTrue(ex.getCause() instanceof IllegalStateException);
-        
+        assertTrue(ex.getCause() instanceof OoopsieRuntimeException);
+
         List<ExceptionEvent> auditlog = this.server.getAuditLog();
         assertNotNull(auditlog);
         assertEquals(1, auditlog.size());
-        
+
         // I/O reactor shut down itself
         assertEquals(IOReactorStatus.SHUT_DOWN, this.server.getStatus());
-        
+
         this.client.shutdown();
         this.server.shutdown();
     }
 
     public void testUnhandledRuntimeException() throws Exception {
 
-        final RequestCount requestConns = new RequestCount(1); 
-        
+        final CountDownLatch requestConns = new CountDownLatch(1);
+
         HttpRequestHandler requestHandler = new HttpRequestHandler() {
 
             public void handle(
-                    final HttpRequest request, 
-                    final HttpResponse response, 
+                    final HttpRequest request,
+                    final HttpResponse response,
                     final HttpContext context) throws HttpException, IOException {
-                throw new IllegalStateException("Oppsie!!!");
+                throw new OoopsieRuntimeException();
             }
-            
+
         };
-        
+
         HttpRequestExecutionHandler requestExecutionHandler = new HttpRequestExecutionHandler() {
 
             public void initalizeContext(final HttpContext context, final Object attachment) {
@@ -291,12 +371,12 @@ public class TestDefaultIOReactors extends HttpCoreNIOTestBase {
                     return null;
                 }
             }
-            
+
             public void handleResponse(final HttpResponse response, final HttpContext context) {
             }
-            
+
         };
-     
+
         IOReactorExceptionHandler exceptionHandler = new IOReactorExceptionHandler() {
 
             public boolean handle(final IOException ex) {
@@ -304,71 +384,92 @@ public class TestDefaultIOReactors extends HttpCoreNIOTestBase {
             }
 
             public boolean handle(final RuntimeException ex) {
-                requestConns.decrement();                    
+                requestConns.countDown();
                 return false;
             }
-          
+
         };
-        
-        NHttpServiceHandler serviceHandler = createHttpServiceHandler(
-                requestHandler, 
-                null,
-                new SimpleEventListener());
-        
-        NHttpClientHandler clientHandler = createHttpClientHandler(
-                requestExecutionHandler,
+
+        HttpProcessor serverHttpProc = new ImmutableHttpProcessor(new HttpResponseInterceptor[] {
+                new ResponseDate(),
+                new ResponseServer(),
+                new ResponseContent(),
+                new ResponseConnControl()
+        });
+
+        BufferingHttpServiceHandler serviceHandler = new BufferingHttpServiceHandler(
+                serverHttpProc,
+                new DefaultHttpResponseFactory(),
+                new DefaultConnectionReuseStrategy(),
+                this.server.getParams());
+
+        serviceHandler.setHandlerResolver(
+                new SimpleHttpRequestHandlerResolver(requestHandler));
+        serviceHandler.setEventListener(
                 new SimpleEventListener());
 
+        HttpProcessor clientHttpProc = new ImmutableHttpProcessor(new HttpRequestInterceptor[] {
+                new RequestContent(),
+                new RequestTargetHost(),
+                new RequestConnControl(),
+                new RequestUserAgent(),
+                new RequestExpectContinue()});
+
+        BufferingHttpClientHandler clientHandler = new BufferingHttpClientHandler(
+                clientHttpProc,
+                requestExecutionHandler,
+                new DefaultConnectionReuseStrategy(),
+                this.client.getParams());
+
         this.server.setExceptionHandler(exceptionHandler);
-        
         this.server.start(serviceHandler);
         this.client.start(clientHandler);
-        
+
         ListenerEndpoint endpoint = this.server.getListenerEndpoint();
         endpoint.waitFor();
         InetSocketAddress serverAddress = (InetSocketAddress) endpoint.getAddress();
-        
+
         this.client.openConnection(
-                new InetSocketAddress("localhost", serverAddress.getPort()), 
+                new InetSocketAddress("localhost", serverAddress.getPort()),
                 null);
-     
-        requestConns.await(10000);
-        assertEquals(0, requestConns.getValue());
-        
+
+        requestConns.await();
+        assertEquals(0, requestConns.getCount());
+
         this.server.join(20000);
-        
+
         Exception ex = this.server.getException();
         assertNotNull(ex);
         assertTrue(ex instanceof IOReactorException);
         assertNotNull(ex.getCause());
-        assertTrue(ex.getCause() instanceof IllegalStateException);
+        assertTrue(ex.getCause() instanceof OoopsieRuntimeException);
 
         List<ExceptionEvent> auditlog = this.server.getAuditLog();
         assertNotNull(auditlog);
         assertEquals(1, auditlog.size());
-        
+
         // I/O reactor shut down itself
         assertEquals(IOReactorStatus.SHUT_DOWN, this.server.getStatus());
-        
+
         this.client.shutdown();
         this.server.shutdown();
     }
 
     public void testHandledRuntimeException() throws Exception {
 
-        final RequestCount requestConns = new RequestCount(1); 
-        
+        final CountDownLatch requestConns = new CountDownLatch(1);
+
         HttpRequestHandler requestHandler = new HttpRequestHandler() {
 
             public void handle(
-                    final HttpRequest request, 
-                    final HttpResponse response, 
+                    final HttpRequest request,
+                    final HttpResponse response,
                     final HttpContext context) throws HttpException, IOException {
-                throw new IllegalStateException("Oppsie!!!");
+                throw new OoopsieRuntimeException();
             }
-            
+
         };
-        
+
         HttpRequestExecutionHandler requestExecutionHandler = new HttpRequestExecutionHandler() {
 
             public void initalizeContext(final HttpContext context, final Object attachment) {
@@ -387,12 +488,12 @@ public class TestDefaultIOReactors extends HttpCoreNIOTestBase {
                     return null;
                 }
             }
-            
+
             public void handleResponse(final HttpResponse response, final HttpContext context) {
             }
-            
+
         };
-     
+
         IOReactorExceptionHandler exceptionHandler = new IOReactorExceptionHandler() {
 
             public boolean handle(final IOException ex) {
@@ -400,42 +501,64 @@ public class TestDefaultIOReactors extends HttpCoreNIOTestBase {
             }
 
             public boolean handle(final RuntimeException ex) {
-                requestConns.decrement();                    
+                requestConns.countDown();
                 return true;
             }
-          
+
         };
-        
-        NHttpServiceHandler serviceHandler = createHttpServiceHandler(
-                requestHandler, 
-                null,
-                new SimpleEventListener());
-        
-        NHttpClientHandler clientHandler = createHttpClientHandler(
-                requestExecutionHandler,
+
+        HttpProcessor serverHttpProc = new ImmutableHttpProcessor(new HttpResponseInterceptor[] {
+                new ResponseDate(),
+                new ResponseServer(),
+                new ResponseContent(),
+                new ResponseConnControl()
+        });
+
+        BufferingHttpServiceHandler serviceHandler = new BufferingHttpServiceHandler(
+                serverHttpProc,
+                new DefaultHttpResponseFactory(),
+                new DefaultConnectionReuseStrategy(),
+                this.server.getParams());
+
+        serviceHandler.setHandlerResolver(
+                new SimpleHttpRequestHandlerResolver(requestHandler));
+        serviceHandler.setEventListener(
                 new SimpleEventListener());
 
+        HttpProcessor clientHttpProc = new ImmutableHttpProcessor(new HttpRequestInterceptor[] {
+                new RequestContent(),
+                new RequestTargetHost(),
+                new RequestConnControl(),
+                new RequestUserAgent(),
+                new RequestExpectContinue()});
+
+        BufferingHttpClientHandler clientHandler = new BufferingHttpClientHandler(
+                clientHttpProc,
+                requestExecutionHandler,
+                new DefaultConnectionReuseStrategy(),
+                this.client.getParams());
+
         this.server.setExceptionHandler(exceptionHandler);
-        
+
         this.server.start(serviceHandler);
         this.client.start(clientHandler);
-        
+
         ListenerEndpoint endpoint = this.server.getListenerEndpoint();
         endpoint.waitFor();
         InetSocketAddress serverAddress = (InetSocketAddress) endpoint.getAddress();
-        
+
         this.client.openConnection(
-                new InetSocketAddress("localhost", serverAddress.getPort()), 
+                new InetSocketAddress("localhost", serverAddress.getPort()),
                 null);
-     
-        requestConns.await(10000);
-        assertEquals(0, requestConns.getValue());
-        
+
+        requestConns.await();
+        assertEquals(0, requestConns.getCount());
+
         this.server.join(1000);
-        
+
         assertEquals(IOReactorStatus.ACTIVE, this.server.getStatus());
         assertNull(this.server.getException());
-        
+
         this.client.shutdown();
         this.server.shutdown();
     }
