@@ -1,7 +1,4 @@
 /*
- * $HeadURL: https://svn.apache.org/repos/asf/httpcomponents/httpcore/tags/4.0.1/httpcore/src/test/java/org/apache/http/impl/TestSessionBuffers.java $
- * $Revision: 674585 $
- * $Date: 2008-07-07 20:42:42 +0200 (Mon, 07 Jul 2008) $
  * ====================================================================
  * Licensed to the Apache Software Foundation (ASF) under one
  * or more contributor license agreements.  See the NOTICE file
@@ -35,14 +32,12 @@ import java.io.ByteArrayOutputStream;
 import java.io.IOException;
 import java.io.InputStream;
 
-import junit.framework.Test;
 import junit.framework.TestCase;
-import junit.framework.TestSuite;
-import org.apache.http.io.HttpTransportMetrics;
 
-import org.apache.http.params.BasicHttpParams;
+import org.apache.http.io.HttpTransportMetrics;
 import org.apache.http.mockup.SessionInputBufferMockup;
 import org.apache.http.mockup.SessionOutputBufferMockup;
+import org.apache.http.params.BasicHttpParams;
 import org.apache.http.params.CoreConnectionPNames;
 import org.apache.http.params.HttpParams;
 import org.apache.http.params.HttpProtocolParams;
@@ -55,51 +50,57 @@ public class TestSessionBuffers extends TestCase {
         super(testName);
     }
 
-    // ------------------------------------------------------- TestCase Methods
-
-    public static Test suite() {
-        return new TestSuite(TestSessionBuffers.class);
-    }
-
-    // ------------------------------------------------------------------- Main
-    public static void main(String args[]) {
-        String[] testCaseName = { TestSessionBuffers.class.getName() };
-        junit.textui.TestRunner.main(testCaseName);
-    }
-
     public void testInit() throws Exception {
         ByteArrayOutputStream out = new ByteArrayOutputStream();
-        new SessionOutputBufferMockup(out); 
+        new SessionOutputBufferMockup(out);
         try {
-            new SessionOutputBufferMockup(null, new BasicHttpParams()); 
+            new SessionOutputBufferMockup(null, new BasicHttpParams());
             fail("IllegalArgumentException should have been thrown");
         } catch (IllegalArgumentException ex) {
             //expected
         }
         ByteArrayInputStream in = new ByteArrayInputStream(out.toByteArray());
-        new SessionInputBufferMockup(in, 10); 
+        new SessionInputBufferMockup(in, 10);
         try {
-            new SessionInputBufferMockup(in, -10); 
+            new SessionInputBufferMockup(in, -10);
             fail("IllegalArgumentException should have been thrown");
         } catch (IllegalArgumentException ex) {
             //expected
         }
         try {
-            new SessionOutputBufferMockup(out, -10); 
+            new SessionOutputBufferMockup(out, -10);
             fail("IllegalArgumentException should have been thrown");
         } catch (IllegalArgumentException ex) {
             //expected
         }
         try {
-            new SessionInputBufferMockup((InputStream)null, 1024); 
+            new SessionInputBufferMockup((InputStream)null, 1024);
             fail("IllegalArgumentException should have been thrown");
         } catch (IllegalArgumentException ex) {
             //expected
         }
     }
-    
+
+    public void testBasicBufferProperties() throws Exception {
+        SessionInputBufferMockup inbuffer = new SessionInputBufferMockup(new byte[] { 1, 2 , 3});
+        assertEquals(SessionInputBufferMockup.BUFFER_SIZE, inbuffer.capacity());
+        assertEquals(SessionInputBufferMockup.BUFFER_SIZE, inbuffer.available());
+        assertEquals(0, inbuffer.length());
+        inbuffer.read();
+        assertEquals(SessionInputBufferMockup.BUFFER_SIZE - 2, inbuffer.available());
+        assertEquals(2, inbuffer.length());
+
+        SessionOutputBufferMockup outbuffer = new SessionOutputBufferMockup();
+        assertEquals(SessionOutputBufferMockup.BUFFER_SIZE, outbuffer.capacity());
+        assertEquals(SessionOutputBufferMockup.BUFFER_SIZE, outbuffer.available());
+        assertEquals(0, outbuffer.length());
+        outbuffer.write(new byte[] {1, 2, 3});
+        assertEquals(SessionOutputBufferMockup.BUFFER_SIZE - 3, outbuffer.available());
+        assertEquals(3, outbuffer.length());
+    }
+
     public void testBasicReadWriteLine() throws Exception {
-        
+
         String[] teststrs = new String[5];
         teststrs[0] = "Hello";
         teststrs[1] = "This string should be much longer than the size of the output buffer " +
@@ -112,9 +113,9 @@ public class TestSessionBuffers extends TestCase {
         teststrs[2] = buffer.toString();
         teststrs[3] = "";
         teststrs[4] = "And goodbye";
-        
-        CharArrayBuffer chbuffer = new CharArrayBuffer(16); 
-        SessionOutputBufferMockup outbuffer = new SessionOutputBufferMockup(); 
+
+        CharArrayBuffer chbuffer = new CharArrayBuffer(16);
+        SessionOutputBufferMockup outbuffer = new SessionOutputBufferMockup();
         for (int i = 0; i < teststrs.length; i++) {
             chbuffer.clear();
             chbuffer.append(teststrs[i]);
@@ -124,7 +125,7 @@ public class TestSessionBuffers extends TestCase {
         outbuffer.writeLine((String)null);
         outbuffer.writeLine((CharArrayBuffer)null);
         outbuffer.flush();
-        
+
         HttpTransportMetrics tmetrics = outbuffer.getMetrics();
         long bytesWritten = tmetrics.getBytesTransferred();
         long expected = 0;
@@ -132,14 +133,14 @@ public class TestSessionBuffers extends TestCase {
             expected += (teststrs[i].length() + 2/*CRLF*/);
         }
         assertEquals(expected, bytesWritten);
-        
+
         SessionInputBufferMockup inbuffer = new SessionInputBufferMockup(
                 outbuffer.getData());
 
         for (int i = 0; i < teststrs.length; i++) {
             assertEquals(teststrs[i], inbuffer.readLine());
         }
-        
+
         assertNull(inbuffer.readLine());
         assertNull(inbuffer.readLine());
         tmetrics = inbuffer.getMetrics();
@@ -148,7 +149,7 @@ public class TestSessionBuffers extends TestCase {
     }
 
     public void testComplexReadWriteLine() throws Exception {
-        SessionOutputBufferMockup outbuffer = new SessionOutputBufferMockup(); 
+        SessionOutputBufferMockup outbuffer = new SessionOutputBufferMockup();
         outbuffer.write(new byte[] {'a', '\n'});
         outbuffer.write(new byte[] {'\r', '\n'});
         outbuffer.write(new byte[] {'\r', '\r', '\n'});
@@ -156,12 +157,12 @@ public class TestSessionBuffers extends TestCase {
         //these write operations should have no effect
         outbuffer.write(null);
         outbuffer.write(null, 0, 12);
-        
+
         outbuffer.flush();
 
         long bytesWritten = outbuffer.getMetrics().getBytesTransferred();
         assertEquals(8, bytesWritten);
-        
+
         StringBuffer buffer = new StringBuffer();
         for (int i = 0; i < 14; i++) {
             buffer.append("a");
@@ -199,7 +200,7 @@ public class TestSessionBuffers extends TestCase {
         outbuffer.flush();
         bytesWritten = outbuffer.getMetrics().getBytesTransferred();
         assertEquals(8 + 14 + 2 + 15 + 2 + 16 + 2 + 1, bytesWritten);
-        
+
         SessionInputBufferMockup inbuffer = new SessionInputBufferMockup(
                 outbuffer.getData());
 
@@ -216,9 +217,9 @@ public class TestSessionBuffers extends TestCase {
         long bytesRead = inbuffer.getMetrics().getBytesTransferred();
         assertEquals(bytesWritten, bytesRead);
     }
-    
+
     public void testBasicReadWriteLineLargeBuffer() throws Exception {
-        
+
         String[] teststrs = new String[5];
         teststrs[0] = "Hello";
         teststrs[1] = "This string should be much longer than the size of the output buffer " +
@@ -231,9 +232,9 @@ public class TestSessionBuffers extends TestCase {
         teststrs[2] = buffer.toString();
         teststrs[3] = "";
         teststrs[4] = "And goodbye";
-        
-        CharArrayBuffer chbuffer = new CharArrayBuffer(16); 
-        SessionOutputBufferMockup outbuffer = new SessionOutputBufferMockup(); 
+
+        CharArrayBuffer chbuffer = new CharArrayBuffer(16);
+        SessionOutputBufferMockup outbuffer = new SessionOutputBufferMockup();
         for (int i = 0; i < teststrs.length; i++) {
             chbuffer.clear();
             chbuffer.append(teststrs[i]);
@@ -243,14 +244,14 @@ public class TestSessionBuffers extends TestCase {
         outbuffer.writeLine((String)null);
         outbuffer.writeLine((CharArrayBuffer)null);
         outbuffer.flush();
-        
+
         long bytesWritten = outbuffer.getMetrics().getBytesTransferred();
         long expected = 0;
         for (int i = 0; i < teststrs.length; i++) {
             expected += (teststrs[i].length() + 2/*CRLF*/);
         }
         assertEquals(expected, bytesWritten);
-        
+
         SessionInputBufferMockup inbuffer = new SessionInputBufferMockup(
                 outbuffer.getData(), 1024);
 
@@ -290,15 +291,15 @@ public class TestSessionBuffers extends TestCase {
         for (int i = 0; i < out.length; i++) {
             assertEquals(out[i], tmp[i]);
         }
-        
+
         SessionInputBufferMockup inbuffer = new SessionInputBufferMockup(tmp);
 
         // these read operations will have no effect
         assertEquals(0, inbuffer.read(null, 0, 10));
-        assertEquals(0, inbuffer.read(null));        
+        assertEquals(0, inbuffer.read(null));
         long bytesRead = inbuffer.getMetrics().getBytesTransferred();
         assertEquals(0, bytesRead);
-        
+
         byte[] in = new byte[40];
         off = 0;
         remaining = in.length;
@@ -322,7 +323,7 @@ public class TestSessionBuffers extends TestCase {
         bytesRead = inbuffer.getMetrics().getBytesTransferred();
         assertEquals(out.length, bytesRead);
     }
-    
+
     public void testReadWriteByte() throws Exception {
         // make the buffer larger than that of outbuffer
         byte[] out = new byte[40];
@@ -342,7 +343,7 @@ public class TestSessionBuffers extends TestCase {
         for (int i = 0; i < out.length; i++) {
             assertEquals(out[i], tmp[i]);
         }
-        
+
         SessionInputBufferMockup inbuffer = new SessionInputBufferMockup(tmp);
         byte[] in = new byte[40];
         for (int i = 0; i < in.length; i++) {
@@ -360,14 +361,14 @@ public class TestSessionBuffers extends TestCase {
     public void testLineLimit() throws Exception {
         HttpParams params = new BasicHttpParams();
         String s = "a very looooooooooooooooooooooooooooooooooooooong line\r\n     ";
-        byte[] tmp = s.getBytes("US-ASCII"); 
+        byte[] tmp = s.getBytes("US-ASCII");
         // no limit
         params.setIntParameter(CoreConnectionPNames.MAX_LINE_LENGTH, 0);
         SessionInputBufferMockup inbuffer1 = new SessionInputBufferMockup(tmp, 5, params);
         assertNotNull(inbuffer1.readLine());
         long bytesRead = inbuffer1.getMetrics().getBytesTransferred();
         assertEquals(60, bytesRead);
-        
+
         // 15 char limit
         params.setIntParameter(CoreConnectionPNames.MAX_LINE_LENGTH, 15);
         SessionInputBufferMockup inbuffer2 = new SessionInputBufferMockup(tmp, 5, params);
@@ -384,17 +385,17 @@ public class TestSessionBuffers extends TestCase {
     static final int SWISS_GERMAN_HELLO [] = {
         0x47, 0x72, 0xFC, 0x65, 0x7A, 0x69, 0x5F, 0x7A, 0xE4, 0x6D, 0xE4
     };
-        
+
     static final int RUSSIAN_HELLO [] = {
-        0x412, 0x441, 0x435, 0x43C, 0x5F, 0x43F, 0x440, 0x438, 
-        0x432, 0x435, 0x442 
-    }; 
-    
+        0x412, 0x441, 0x435, 0x43C, 0x5F, 0x43F, 0x440, 0x438,
+        0x432, 0x435, 0x442
+    };
+
     private static String constructString(int [] unicodeChars) {
         StringBuffer buffer = new StringBuffer();
         if (unicodeChars != null) {
             for (int i = 0; i < unicodeChars.length; i++) {
-                buffer.append((char)unicodeChars[i]); 
+                buffer.append((char)unicodeChars[i]);
             }
         }
         return buffer.toString();
@@ -404,13 +405,13 @@ public class TestSessionBuffers extends TestCase {
         String s1 = constructString(SWISS_GERMAN_HELLO);
         String s2 = constructString(RUSSIAN_HELLO);
         String s3 = "Like hello and stuff";
-        
+
         HttpParams params = new BasicHttpParams();
         HttpProtocolParams.setHttpElementCharset(params, "UTF-8");
-        
+
         SessionOutputBufferMockup outbuffer = new SessionOutputBufferMockup(params);
 
-        CharArrayBuffer chbuffer = new CharArrayBuffer(16); 
+        CharArrayBuffer chbuffer = new CharArrayBuffer(16);
         for (int i = 0; i < 10; i++) {
             chbuffer.clear();
             chbuffer.append(s1);
@@ -428,7 +429,7 @@ public class TestSessionBuffers extends TestCase {
                 (s2.toString().getBytes("UTF-8").length + 2) +
                 (s3.toString().getBytes("UTF-8").length + 2)) * 10;
         assertEquals(expected, bytesWritten);
-        
+
         SessionInputBufferMockup inbuffer = new SessionInputBufferMockup(
                 outbuffer.getData(), params);
 
@@ -445,13 +446,13 @@ public class TestSessionBuffers extends TestCase {
 
     public void testNonAsciiReadWriteLine() throws Exception {
         String s1 = constructString(SWISS_GERMAN_HELLO);
-        
+
         HttpParams params = new BasicHttpParams();
         HttpProtocolParams.setHttpElementCharset(params, HTTP.ISO_8859_1);
-        
+
         SessionOutputBufferMockup outbuffer = new SessionOutputBufferMockup(params);
 
-        CharArrayBuffer chbuffer = new CharArrayBuffer(16); 
+        CharArrayBuffer chbuffer = new CharArrayBuffer(16);
         for (int i = 0; i < 10; i++) {
             chbuffer.clear();
             chbuffer.append(s1);
@@ -461,14 +462,17 @@ public class TestSessionBuffers extends TestCase {
         long bytesWritten = outbuffer.getMetrics().getBytesTransferred();
         long expected = ((s1.toString().getBytes(HTTP.ISO_8859_1).length + 2)) * 10;
         assertEquals(expected, bytesWritten);
-        
+
         SessionInputBufferMockup inbuffer = new SessionInputBufferMockup(
                 outbuffer.getData(),
                 params);
         HttpProtocolParams.setHttpElementCharset(params, HTTP.ISO_8859_1);
 
         for (int i = 0; i < 10; i++) {
-            assertEquals(s1, inbuffer.readLine());
+            CharArrayBuffer buf = new CharArrayBuffer(64);
+            int len = inbuffer.readLine(buf);
+            assertEquals(len, SWISS_GERMAN_HELLO.length);
+            assertEquals(s1, buf.toString());
         }
         assertNull(inbuffer.readLine());
         assertNull(inbuffer.readLine());
@@ -479,7 +483,7 @@ public class TestSessionBuffers extends TestCase {
     public void testInvalidCharArrayBuffer() throws Exception {
         SessionInputBufferMockup inbuffer = new SessionInputBufferMockup(new byte[] {});
         try {
-            inbuffer.readLine(null); 
+            inbuffer.readLine(null);
             fail("IllegalArgumentException should have been thrown");
         } catch (IllegalArgumentException ex) {
             //expected
@@ -487,6 +491,6 @@ public class TestSessionBuffers extends TestCase {
             assertEquals(0, bytesRead);
         }
     }
-    
+
 }
 
