@@ -1,7 +1,4 @@
 /*
- * $HeadURL: https://svn.apache.org/repos/asf/httpcomponents/httpcore/tags/4.0.1/httpcore/src/test/java/org/apache/http/protocol/TestHttpServiceAndExecutor.java $
- * $Revision: 613298 $
- * $Date: 2008-01-18 23:09:22 +0100 (Fri, 18 Jan 2008) $
  * ====================================================================
  * Licensed to the Apache Software Foundation (ASF) under one
  * or more contributor license agreements.  See the NOTICE file
@@ -30,12 +27,19 @@
 
 package org.apache.http.protocol;
 
+import java.io.BufferedReader;
 import java.io.IOException;
+import java.io.InputStream;
+import java.io.InputStreamReader;
+import java.io.OutputStream;
+import java.io.UnsupportedEncodingException;
 import java.net.Socket;
 import java.util.ArrayList;
 import java.util.List;
 import java.util.Random;
 
+import junit.framework.TestCase;
+
 import org.apache.http.Header;
 import org.apache.http.HttpEntity;
 import org.apache.http.HttpEntityEnclosingRequest;
@@ -45,18 +49,18 @@ import org.apache.http.HttpRequest;
 import org.apache.http.HttpResponse;
 import org.apache.http.HttpStatus;
 import org.apache.http.HttpVersion;
+import org.apache.http.entity.AbstractHttpEntity;
 import org.apache.http.entity.ByteArrayEntity;
 import org.apache.http.entity.StringEntity;
 import org.apache.http.impl.DefaultHttpClientConnection;
 import org.apache.http.message.BasicHttpEntityEnclosingRequest;
 import org.apache.http.message.BasicHttpRequest;
-import org.apache.http.mockup.TestHttpClient;
-import org.apache.http.mockup.TestHttpServer;
+import org.apache.http.mockup.HttpClient;
+import org.apache.http.mockup.HttpServer;
 import org.apache.http.params.CoreProtocolPNames;
 import org.apache.http.util.EncodingUtils;
 import org.apache.http.util.EntityUtils;
 
-import junit.framework.*;
 import org.apache.http.HttpConnectionMetrics;
 
 public class TestHttpServiceAndExecutor extends TestCase {
@@ -66,24 +70,14 @@ public class TestHttpServiceAndExecutor extends TestCase {
         super(testName);
     }
 
-    // ------------------------------------------------------------------- Main
-    public static void main(String args[]) {
-        String[] testCaseName = { TestHttpServiceAndExecutor.class.getName() };
-        junit.textui.TestRunner.main(testCaseName);
-    }
-
     // ------------------------------------------------------- TestCase Methods
 
-    public static Test suite() {
-        return new TestSuite(TestHttpServiceAndExecutor.class);
-    }
+    private HttpServer server;
+    private HttpClient client;
 
-    private TestHttpServer server;
-    private TestHttpClient client;
-    
     protected void setUp() throws Exception {
-        this.server = new TestHttpServer();
-        this.client = new TestHttpClient();
+        this.server = new HttpServer();
+        this.client = new HttpClient();
     }
 
     protected void tearDown() throws Exception {
@@ -91,14 +85,14 @@ public class TestHttpServiceAndExecutor extends TestCase {
     }
 
     /**
-     * This test case executes a series of simple GET requests 
+     * This test case executes a series of simple GET requests
      */
     public void testSimpleBasicHttpRequests() throws Exception {
-        
+
         int reqNo = 20;
-        
+
         Random rnd = new Random();
-        
+
         // Prepare some random data
         final List testData = new ArrayList(reqNo);
         for (int i = 0; i < reqNo; i++) {
@@ -112,39 +106,39 @@ public class TestHttpServiceAndExecutor extends TestCase {
         this.server.registerHandler("*", new HttpRequestHandler() {
 
             public void handle(
-                    final HttpRequest request, 
-                    final HttpResponse response, 
+                    final HttpRequest request,
+                    final HttpResponse response,
                     final HttpContext context) throws HttpException, IOException {
-                
+
                 String s = request.getRequestLine().getUri();
                 if (s.startsWith("/?")) {
                     s = s.substring(2);
                 }
                 int index = Integer.parseInt(s);
                 byte[] data = (byte []) testData.get(index);
-                ByteArrayEntity entity = new ByteArrayEntity(data); 
+                ByteArrayEntity entity = new ByteArrayEntity(data);
                 response.setEntity(entity);
             }
-            
+
         });
-        
+
         this.server.start();
-        
+
         DefaultHttpClientConnection conn = new DefaultHttpClientConnection();
         HttpHost host = new HttpHost("localhost", this.server.getPort());
-        
+
         try {
             for (int r = 0; r < reqNo; r++) {
                 if (!conn.isOpen()) {
                     Socket socket = new Socket(host.getHostName(), host.getPort());
                     conn.bind(socket, this.client.getParams());
                 }
-                
+
                 BasicHttpRequest get = new BasicHttpRequest("GET", "/?" + r);
                 HttpResponse response = this.client.execute(get, host, conn);
                 byte[] received = EntityUtils.toByteArray(response.getEntity());
                 byte[] expected = (byte[]) testData.get(r);
-                
+
                 assertEquals(expected.length, received.length);
                 for (int i = 0; i < expected.length; i++) {
                     assertEquals(expected[i], received[i]);
@@ -153,12 +147,12 @@ public class TestHttpServiceAndExecutor extends TestCase {
                     conn.close();
                 }
             }
-            
+
             //Verify the connection metrics
             HttpConnectionMetrics cm = conn.getMetrics();
             assertEquals(reqNo, cm.getRequestCount());
             assertEquals(reqNo, cm.getResponseCount());
-            
+
         } finally {
             conn.close();
             this.server.shutdown();
@@ -166,15 +160,15 @@ public class TestHttpServiceAndExecutor extends TestCase {
     }
 
     /**
-     * This test case executes a series of simple POST requests with content length 
-     * delimited content. 
+     * This test case executes a series of simple POST requests with content length
+     * delimited content.
      */
     public void testSimpleHttpPostsWithContentLength() throws Exception {
-        
+
         int reqNo = 20;
-        
+
         Random rnd = new Random();
-        
+
         // Prepare some random data
         List testData = new ArrayList(reqNo);
         for (int i = 0; i < reqNo; i++) {
@@ -188,37 +182,37 @@ public class TestHttpServiceAndExecutor extends TestCase {
         this.server.registerHandler("*", new HttpRequestHandler() {
 
             public void handle(
-                    final HttpRequest request, 
-                    final HttpResponse response, 
+                    final HttpRequest request,
+                    final HttpResponse response,
                     final HttpContext context) throws HttpException, IOException {
-                
+
                 if (request instanceof HttpEntityEnclosingRequest) {
                     HttpEntity incoming = ((HttpEntityEnclosingRequest) request).getEntity();
                     byte[] data = EntityUtils.toByteArray(incoming);
-                    
+
                     ByteArrayEntity outgoing = new ByteArrayEntity(data);
                     outgoing.setChunked(false);
                     response.setEntity(outgoing);
                 } else {
-                    StringEntity outgoing = new StringEntity("No content"); 
+                    StringEntity outgoing = new StringEntity("No content");
                     response.setEntity(outgoing);
                 }
             }
-            
+
         });
-        
+
         this.server.start();
-        
+
         DefaultHttpClientConnection conn = new DefaultHttpClientConnection();
         HttpHost host = new HttpHost("localhost", this.server.getPort());
-        
+
         try {
             for (int r = 0; r < reqNo; r++) {
                 if (!conn.isOpen()) {
                     Socket socket = new Socket(host.getHostName(), host.getPort());
                     conn.bind(socket, this.client.getParams());
                 }
-                
+
                 BasicHttpEntityEnclosingRequest post = new BasicHttpEntityEnclosingRequest("POST", "/");
                 byte[] data = (byte[]) testData.get(r);
                 ByteArrayEntity outgoing = new ByteArrayEntity(data);
@@ -227,7 +221,7 @@ public class TestHttpServiceAndExecutor extends TestCase {
                 HttpResponse response = this.client.execute(post, host, conn);
                 byte[] received = EntityUtils.toByteArray(response.getEntity());
                 byte[] expected = (byte[]) testData.get(r);
-                
+
                 assertEquals(expected.length, received.length);
                 for (int i = 0; i < expected.length; i++) {
                     assertEquals(expected[i], received[i]);
@@ -240,7 +234,7 @@ public class TestHttpServiceAndExecutor extends TestCase {
             HttpConnectionMetrics cm = conn.getMetrics();
             assertEquals(reqNo, cm.getRequestCount());
             assertEquals(reqNo, cm.getResponseCount());
-            
+
         } finally {
             conn.close();
             this.server.shutdown();
@@ -248,15 +242,15 @@ public class TestHttpServiceAndExecutor extends TestCase {
     }
 
     /**
-     * This test case executes a series of simple POST requests with chunk 
-     * coded content content. 
+     * This test case executes a series of simple POST requests with chunk
+     * coded content content.
      */
     public void testSimpleHttpPostsChunked() throws Exception {
-        
+
         int reqNo = 20;
-        
+
         Random rnd = new Random();
-        
+
         // Prepare some random data
         List testData = new ArrayList(reqNo);
         for (int i = 0; i < reqNo; i++) {
@@ -270,37 +264,37 @@ public class TestHttpServiceAndExecutor extends TestCase {
         this.server.registerHandler("*", new HttpRequestHandler() {
 
             public void handle(
-                    final HttpRequest request, 
-                    final HttpResponse response, 
+                    final HttpRequest request,
+                    final HttpResponse response,
                     final HttpContext context) throws HttpException, IOException {
-                
+
                 if (request instanceof HttpEntityEnclosingRequest) {
                     HttpEntity incoming = ((HttpEntityEnclosingRequest) request).getEntity();
                     byte[] data = EntityUtils.toByteArray(incoming);
-                    
+
                     ByteArrayEntity outgoing = new ByteArrayEntity(data);
                     outgoing.setChunked(true);
                     response.setEntity(outgoing);
                 } else {
-                    StringEntity outgoing = new StringEntity("No content"); 
+                    StringEntity outgoing = new StringEntity("No content");
                     response.setEntity(outgoing);
                 }
             }
-            
+
         });
-        
+
         this.server.start();
-        
+
         DefaultHttpClientConnection conn = new DefaultHttpClientConnection();
         HttpHost host = new HttpHost("localhost", this.server.getPort());
-        
+
         try {
             for (int r = 0; r < reqNo; r++) {
                 if (!conn.isOpen()) {
                     Socket socket = new Socket(host.getHostName(), host.getPort());
                     conn.bind(socket, this.client.getParams());
                 }
-                
+
                 BasicHttpEntityEnclosingRequest post = new BasicHttpEntityEnclosingRequest("POST", "/");
                 byte[] data = (byte[]) testData.get(r);
                 ByteArrayEntity outgoing = new ByteArrayEntity(data);
@@ -310,7 +304,7 @@ public class TestHttpServiceAndExecutor extends TestCase {
                 HttpResponse response = this.client.execute(post, host, conn);
                 byte[] received = EntityUtils.toByteArray(response.getEntity());
                 byte[] expected = (byte[]) testData.get(r);
-                
+
                 assertEquals(expected.length, received.length);
                 for (int i = 0; i < expected.length; i++) {
                     assertEquals(expected[i], received[i]);
@@ -330,14 +324,14 @@ public class TestHttpServiceAndExecutor extends TestCase {
     }
 
     /**
-     * This test case executes a series of simple HTTP/1.0 POST requests. 
+     * This test case executes a series of simple HTTP/1.0 POST requests.
      */
     public void testSimpleHttpPostsHTTP10() throws Exception {
-        
+
         int reqNo = 20;
-        
+
         Random rnd = new Random();
-        
+
         // Prepare some random data
         List testData = new ArrayList(reqNo);
         for (int i = 0; i < reqNo; i++) {
@@ -351,41 +345,41 @@ public class TestHttpServiceAndExecutor extends TestCase {
         this.server.registerHandler("*", new HttpRequestHandler() {
 
             public void handle(
-                    final HttpRequest request, 
-                    final HttpResponse response, 
+                    final HttpRequest request,
+                    final HttpResponse response,
                     final HttpContext context) throws HttpException, IOException {
-                
+
                 if (request instanceof HttpEntityEnclosingRequest) {
                     HttpEntity incoming = ((HttpEntityEnclosingRequest) request).getEntity();
                     byte[] data = EntityUtils.toByteArray(incoming);
-                    
+
                     ByteArrayEntity outgoing = new ByteArrayEntity(data);
                     outgoing.setChunked(false);
                     response.setEntity(outgoing);
                 } else {
-                    StringEntity outgoing = new StringEntity("No content"); 
+                    StringEntity outgoing = new StringEntity("No content");
                     response.setEntity(outgoing);
                 }
             }
-            
+
         });
-        
+
         this.server.start();
-        
+
         // Set protocol level to HTTP/1.0
         this.client.getParams().setParameter(
                 CoreProtocolPNames.PROTOCOL_VERSION, HttpVersion.HTTP_1_0);
-        
+
         DefaultHttpClientConnection conn = new DefaultHttpClientConnection();
         HttpHost host = new HttpHost("localhost", this.server.getPort());
-        
+
         try {
             for (int r = 0; r < reqNo; r++) {
                 if (!conn.isOpen()) {
                     Socket socket = new Socket(host.getHostName(), host.getPort());
                     conn.bind(socket, this.client.getParams());
                 }
-                
+
                 BasicHttpEntityEnclosingRequest post = new BasicHttpEntityEnclosingRequest("POST", "/");
                 byte[] data = (byte[]) testData.get(r);
                 ByteArrayEntity outgoing = new ByteArrayEntity(data);
@@ -395,7 +389,7 @@ public class TestHttpServiceAndExecutor extends TestCase {
                 assertEquals(HttpVersion.HTTP_1_0, response.getStatusLine().getProtocolVersion());
                 byte[] received = EntityUtils.toByteArray(response.getEntity());
                 byte[] expected = (byte[]) testData.get(r);
-                
+
                 assertEquals(expected.length, received.length);
                 for (int i = 0; i < expected.length; i++) {
                     assertEquals(expected[i], received[i]);
@@ -404,7 +398,7 @@ public class TestHttpServiceAndExecutor extends TestCase {
                     conn.close();
                 }
             }
-            
+
             //Verify the connection metrics
             HttpConnectionMetrics cm = conn.getMetrics();
             assertEquals(reqNo, cm.getRequestCount());
@@ -416,15 +410,15 @@ public class TestHttpServiceAndExecutor extends TestCase {
     }
 
     /**
-     * This test case executes a series of simple POST requests using 
-     * the 'expect: continue' handshake. 
+     * This test case executes a series of simple POST requests using
+     * the 'expect: continue' handshake.
      */
     public void testHttpPostsWithExpectContinue() throws Exception {
-        
+
         int reqNo = 20;
-        
+
         Random rnd = new Random();
-        
+
         // Prepare some random data
         List testData = new ArrayList(reqNo);
         for (int i = 0; i < reqNo; i++) {
@@ -438,40 +432,40 @@ public class TestHttpServiceAndExecutor extends TestCase {
         this.server.registerHandler("*", new HttpRequestHandler() {
 
             public void handle(
-                    final HttpRequest request, 
-                    final HttpResponse response, 
+                    final HttpRequest request,
+                    final HttpResponse response,
                     final HttpContext context) throws HttpException, IOException {
-                
+
                 if (request instanceof HttpEntityEnclosingRequest) {
                     HttpEntity incoming = ((HttpEntityEnclosingRequest) request).getEntity();
                     byte[] data = EntityUtils.toByteArray(incoming);
-                    
+
                     ByteArrayEntity outgoing = new ByteArrayEntity(data);
                     outgoing.setChunked(true);
                     response.setEntity(outgoing);
                 } else {
-                    StringEntity outgoing = new StringEntity("No content"); 
+                    StringEntity outgoing = new StringEntity("No content");
                     response.setEntity(outgoing);
                 }
             }
-            
+
         });
-        
+
         this.server.start();
-        
+
         // Activate 'expect: continue' handshake
         this.client.getParams().setBooleanParameter(CoreProtocolPNames.USE_EXPECT_CONTINUE, true);
-        
+
         DefaultHttpClientConnection conn = new DefaultHttpClientConnection();
         HttpHost host = new HttpHost("localhost", this.server.getPort());
-        
+
         try {
             for (int r = 0; r < reqNo; r++) {
                 if (!conn.isOpen()) {
                     Socket socket = new Socket(host.getHostName(), host.getPort());
                     conn.bind(socket, this.client.getParams());
                 }
-                
+
                 BasicHttpEntityEnclosingRequest post = new BasicHttpEntityEnclosingRequest("POST", "/");
                 byte[] data = (byte[]) testData.get(r);
                 ByteArrayEntity outgoing = new ByteArrayEntity(data);
@@ -481,7 +475,7 @@ public class TestHttpServiceAndExecutor extends TestCase {
                 HttpResponse response = this.client.execute(post, host, conn);
                 byte[] received = EntityUtils.toByteArray(response.getEntity());
                 byte[] expected = (byte[]) testData.get(r);
-                
+
                 assertEquals(expected.length, received.length);
                 for (int i = 0; i < expected.length; i++) {
                     assertEquals(expected[i], received[i]);
@@ -490,7 +484,7 @@ public class TestHttpServiceAndExecutor extends TestCase {
                     conn.close();
                 }
             }
-            
+
             //Verify the connection metrics
             HttpConnectionMetrics cm = conn.getMetrics();
             assertEquals(reqNo, cm.getRequestCount());
@@ -500,35 +494,35 @@ public class TestHttpServiceAndExecutor extends TestCase {
             this.server.shutdown();
         }
     }
-    
-    
+
+
     /**
-     * This test case executes a series of simple POST requests that do not 
-     * meet the target server expectations. 
+     * This test case executes a series of simple POST requests that do not
+     * meet the target server expectations.
      */
     public void testHttpPostsWithExpectationVerification() throws Exception {
-        
+
         int reqNo = 3;
-        
+
         // Initialize the server-side request handler
         this.server.registerHandler("*", new HttpRequestHandler() {
 
             public void handle(
-                    final HttpRequest request, 
-                    final HttpResponse response, 
+                    final HttpRequest request,
+                    final HttpResponse response,
                     final HttpContext context) throws HttpException, IOException {
-                
-                StringEntity outgoing = new StringEntity("No content"); 
+
+                StringEntity outgoing = new StringEntity("No content");
                 response.setEntity(outgoing);
             }
-            
+
         });
-        
+
         this.server.setExpectationVerifier(new HttpExpectationVerifier() {
 
             public void verify(
-                    final HttpRequest request, 
-                    final HttpResponse response, 
+                    final HttpRequest request,
+                    final HttpResponse response,
                     final HttpContext context) throws HttpException {
                 Header someheader = request.getFirstHeader("Secret");
                 if (someheader != null) {
@@ -542,47 +536,47 @@ public class TestHttpServiceAndExecutor extends TestCase {
                     if (secretNumber < 2) {
                         response.setStatusCode(HttpStatus.SC_EXPECTATION_FAILED);
                         ByteArrayEntity outgoing = new ByteArrayEntity(
-                                EncodingUtils.getAsciiBytes("Wrong secret number")); 
+                                EncodingUtils.getAsciiBytes("Wrong secret number"));
                         response.setEntity(outgoing);
                     }
                 }
             }
-            
+
         });
-        
+
         this.server.start();
-        
+
         // Activate 'expect: continue' handshake
         this.client.getParams().setBooleanParameter(CoreProtocolPNames.USE_EXPECT_CONTINUE, true);
-        
+
         DefaultHttpClientConnection conn = new DefaultHttpClientConnection();
         HttpHost host = new HttpHost("localhost", this.server.getPort());
-        
+
         try {
             for (int r = 0; r < reqNo; r++) {
                 if (!conn.isOpen()) {
                     Socket socket = new Socket(host.getHostName(), host.getPort());
                     conn.bind(socket, this.client.getParams());
                 }
-                
+
                 BasicHttpEntityEnclosingRequest post = new BasicHttpEntityEnclosingRequest("POST", "/");
                 post.addHeader("Secret", Integer.toString(r));
                 ByteArrayEntity outgoing = new ByteArrayEntity(
-                        EncodingUtils.getAsciiBytes("No content")); 
+                        EncodingUtils.getAsciiBytes("No content"));
                 post.setEntity(outgoing);
 
                 HttpResponse response = this.client.execute(post, host, conn);
 
                 HttpEntity entity = response.getEntity();
                 assertNotNull(entity);
-                entity.consumeContent();
-                
+                EntityUtils.consume(entity);
+
                 if (r < 2) {
                     assertEquals(HttpStatus.SC_EXPECTATION_FAILED, response.getStatusLine().getStatusCode());
                 } else {
                     assertEquals(HttpStatus.SC_OK, response.getStatusLine().getStatusCode());
                 }
-                
+
                 if (!this.client.keepAlive(response)) {
                     conn.close();
                 }
@@ -596,5 +590,167 @@ public class TestHttpServiceAndExecutor extends TestCase {
             this.server.shutdown();
         }
     }
-    
+
+    static class RepeatingEntity extends AbstractHttpEntity {
+
+        private final byte[] raw;
+        private int n;
+
+        public RepeatingEntity(final String content, String charset, int n) {
+            super();
+            if (content == null) {
+                throw new IllegalArgumentException("Content may not be null");
+            }
+            if (n <= 0) {
+                throw new IllegalArgumentException("N may not be negative or zero");
+            }
+            if (charset == null) {
+                charset = "US-ASCII";
+            }
+            byte[] b;
+            try {
+                b = content.getBytes(charset);
+            } catch (UnsupportedEncodingException ex) {
+                b = content.getBytes();
+            }
+            this.raw = b;
+            this.n = n;
+        }
+
+        public InputStream getContent() throws IOException, IllegalStateException {
+            throw new IllegalStateException("This method is not implemented");
+        }
+
+        public long getContentLength() {
+            return (this.raw.length + 2) * this.n;
+        }
+
+        public boolean isRepeatable() {
+            return true;
+        }
+
+        public boolean isStreaming() {
+            return false;
+        }
+
+        public void writeTo(final OutputStream outstream) throws IOException {
+            for (int i = 0; i < this.n; i++) {
+                outstream.write(this.raw);
+                outstream.write('\r');
+                outstream.write('\n');
+            }
+            outstream.flush();
+        }
+
+    }
+
+    public void testHttpContent() throws Exception {
+
+        String[] patterns = {
+
+            "0123456789ABCDEF",
+            "yadayada-blahblah-this-and-that-yadayada-blahblah-this-and-that-" +
+            "yadayada-blahblah-this-and-that-yadayada-blahblah-this-and-that-" +
+            "yadayada-blahblah-this-and-that-yadayada-blahblah-this-and-that-" +
+            "yadayada-blahblah-this-and-that-yadayada-blahblah-this-and-that-" +
+            "yadayada-blahblah-this-and-that-yadayada-blahblah-this-and-that-" +
+            "yadayada-blahblah-this-and-that-yadayada-blahblah-this-and-that-" +
+            "yadayada-blahblah-this-and-that-yadayada-blahblah-this-and-that-" +
+            "yadayada-blahblah-this-and-that-yadayada-blahblah-this-and-that-" +
+            "yadayada-blahblah-this-and-that-yadayada-blahblah-this-and-that-" +
+            "yadayada-blahblah-this-and-that-yadayada-blahblah-this-and-that-" +
+            "yadayada-blahblah-this-and-that-yadayada-blahblah-this-and-that-" +
+            "yadayada-blahblah-this-and-that-yadayada-blahblah-this-and-that-" +
+            "yadayada-blahblah-this-and-that-yadayada-blahblah-this-and-that-" +
+            "yadayada-blahblah-this-and-that-yadayada-blahblah-this-and-that-" +
+            "yadayada-blahblah-this-and-that-yadayada-blahblah-this-and-that"
+        };
+
+        // Initialize the server-side request handler
+        this.server.registerHandler("*", new HttpRequestHandler() {
+
+            public void handle(
+                    final HttpRequest request,
+                    final HttpResponse response,
+                    final HttpContext context) throws HttpException, IOException {
+
+                if (request instanceof HttpEntityEnclosingRequest) {
+                    int n = 1;
+                    String s = request.getRequestLine().getUri();
+                    if (s.startsWith("/?n=")) {
+                        s = s.substring(4);
+                        try {
+                            n = Integer.parseInt(s);
+                            if (n <= 0) {
+                                throw new HttpException("Invalid request: " +
+                                        "number of repetitions cannot be negative or zero");
+                            }
+                        } catch (NumberFormatException ex) {
+                            throw new HttpException("Invalid request: " +
+                                    "number of repetitions is invalid");
+                        }
+                    }
+
+                    HttpEntity incoming = ((HttpEntityEnclosingRequest) request).getEntity();
+                    String line = EntityUtils.toString(incoming);
+                    String charset = EntityUtils.getContentCharSet(incoming);
+
+                    RepeatingEntity outgoing = new RepeatingEntity(line, charset, n);
+                    outgoing.setChunked(n % 2 == 0);
+                    response.setEntity(outgoing);
+                } else {
+                    throw new HttpException("Invalid request: POST request expected");
+                }
+            }
+
+        });
+
+        this.server.start();
+        DefaultHttpClientConnection conn = new DefaultHttpClientConnection();
+        HttpHost host = new HttpHost("localhost", this.server.getPort());
+
+        try {
+            for (int i = 0; i < patterns.length; i++) {
+                String pattern = patterns[i];
+                for (int n = 1000; n < 1020; n++) {
+                    if (!conn.isOpen()) {
+                        Socket socket = new Socket(host.getHostName(), host.getPort());
+                        conn.bind(socket, this.client.getParams());
+                    }
+
+                    BasicHttpEntityEnclosingRequest post = new BasicHttpEntityEnclosingRequest(
+                            "POST", "/?n=" + n);
+                    StringEntity outgoing = new StringEntity(pattern);
+                    outgoing.setChunked(n % 2 == 0);
+                    post.setEntity(outgoing);
+
+                    HttpResponse response = this.client.execute(post, host, conn);
+                    HttpEntity incoming = response.getEntity();
+                    assertNotNull(incoming);
+                    InputStream instream = incoming.getContent();
+                    String charset = EntityUtils.getContentCharSet(incoming);
+                    if (charset == null) {
+                        charset = "US-ASCII";
+                    }
+                    assertNotNull(instream);
+                    BufferedReader reader = new BufferedReader(new InputStreamReader(instream, charset));
+
+                    String line;
+                    int count = 0;
+                    while ((line = reader.readLine()) != null) {
+                        assertEquals(pattern, line);
+                        count++;
+                    }
+                    assertEquals(n, count);
+                    if (!this.client.keepAlive(response)) {
+                        conn.close();
+                    }
+                }
+            }
+        } finally {
+            conn.close();
+            this.server.shutdown();
+        }
+    }
+
 }
